# Generated by Django 3.1 on 2022-04-30 12:30

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('app', '0002_auto_20220414_1300'),
    ]

    operations = [
        migrations.CreateModel(
            name='Esewa',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('merchant_key', models.CharField(blank=True, default='epay_payment', max_length=200)),
            ],
        ),
        migrations.CreateModel(
            name='Khalti',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('public_key', models.CharField(blank=True, default='', max_length=200)),
                ('private_key', models.CharField(blank=True, default='', max_length=200)),
            ],
        ),
        migrations.AlterField(
            model_name='usersetting',
            name='verification_expires',
            field=models.DateField(default=datetime.date(2022, 5, 3)),
        ),
    ]



from django.shortcuts import render, redirect
from django.views import View
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from django.http import JsonResponse
from .models import *
from .serializers import RegisterSerializer
from django.contrib.auth.hashers import make_password
from django.contrib import auth
import datetime
import requests
from django.http import JsonResponse
from django.contrib.auth.decorators import login_required
from.models import User
from shop.models import Khalti, eSewa

today = datetime.date.today()

 
def home(request):
	if request.user.is_authenticated:
		return redirect('home/')
	else:
		return render(request, 'index.html')

def index(request):
	user_membership = UserMembership.objects.get(user=request.user)
	subscriptions = Subscription.objects.filter(user_membership=user_membership).exists()
	if subscriptions == False:
		return redirect('sub')
	else:
		subscription = Subscription.objects.filter(user_membership=user_membership).last()
		return render(request, 'home.html', {'sub': subscription})
	

def signin(request):	
	if request.user.is_authenticated:
		return redirect('/')
	else:
		return render(request, 'login.html')

def check_mail_ajax(request):
	if request.is_ajax():
		email = request.GET.get('email', None)
		check_email = User.objects.filter(email=email).exists()
		if check_email == True:
			response = {'error': 'Email already exists.'}
			return JsonResponse(response)
		else:
			response = {'success': 'Cool'}
			return JsonResponse(response)
	else:
		response = {'error': 'Error Email Checking.'}
		return JsonResponse(response)


class Register(APIView):
	def post(self, request):
		serializer = RegisterSerializer(data=request.data)
		if serializer.is_valid():
			serializer.save()
			obj = serializer.save()
			password = make_password(serializer.data['password'])
			User.objects.filter(email=serializer.data['email']).update(password=password)
			get_membership = Membership.objects.get(membership_type='Free')
			instance = UserMembership.objects.create(user=obj, membership=get_membership)
			return Response({'success': 'Registration successful.'})

		else:
			return Response({'error': 'Error. Try again'})


class Login(APIView):
	def post(self, request):
		email = request.data.get('email')
		password = request.data.get('password')

		# Let us check if the user exists or not...
		check_email = User.objects.filter(email=email).exists()
		if check_email == False:
			return Response({'error': 'No account with such email'})
		# We need to check if the user password is correct
		user = User.objects.get(email=email)
		if user.check_password(password) == False:
			return Response({'error': 'Password is not correct. Try again'})
		# Now let us log the user in
		log_user = auth.authenticate(email=email, password=password)
		if user is not None:
			auth.login(request, log_user)
			return Response({'success': 'Login successful'})
		else:
			return Response({'error': 'Invalid email/password. Try again later.'})


def subscription(request):
	return render(request, 'subscription.html')

def end_sub(request):
	return render(request, 'sub.html')


 

def subscribe(request):
	plan = request.GET.get('sub_plan')
	fetch_membership = Membership.objects.filter(membership_type=plan).exists()
	if fetch_membership == False:
		return redirect('subscribe')
	membership = Membership.objects.get(membership_type=plan)
	price = float(membership.price)  
	amount = int(price)
	
	instance = PayHistory.objects.create(amount= amount, payment_for=membership, user=request.user)
	esewa_id = eSewa.objects.get(id=1)
	
	context = {
		'instance': instance,
		'esewa_id':esewa_id
	}
	UserMembership.objects.filter(user=instance.user).update(membership=membership)
	return render(request, 'esewarequest.html',context)

class EsewaVerifyView(View):
    def get(self, request, *args, **kwargs):

        import xml.etree.ElementTree as ET                  

        oid = request.GET.get("oid")                        
        amt = request.GET.get("amt")                        
        refId = request.GET.get("refId")                    

        url = "https://uat.esewa.com.np/epay/transrec"      

        d = {                                               
            'amt': amt,
            'scd': eSewa.objects.get(id=1).merchant_id,
            'rid': refId,
            'pid': oid,
        }
        resp = requests.post(url, d)                        

        root = ET.fromstring(resp.content)                                                 

        status = root[0].text.strip()                       

        order_id = oid 
        order_obj = PayHistory.objects.get(id=order_id)

        if status == "Success":                             
            order_obj.paid = True
            order_obj.save()
            return redirect("/")
        else:                                               
            return redirect("/subscribe/?o_id="+order_id)
                

def subscribed(request):
	return render(request, 'subscribed.html')


def  khaltirequest(request):   
	plan = request.GET.get('sub_plan')
	fetch_membership = Membership.objects.filter(membership_type=plan).exists()
	if fetch_membership == False:
		return redirect('subscribe')
	membership = Membership.objects.get(membership_type=plan)
	price = float(membership.price)  
	amount = int(price)
	instance = PayHistory.objects.create(amount= amount, payment_for=membership, user=request.user)
	khalti = Khalti.objects.get(id=1)
	context = {
		'instance': instance,
		'khalti':khalti
	}
	UserMembership.objects.filter(user=instance.user).update(membership=membership)
	return render(request,"khaltirequest.html", context)


class KhaltiVerifyView(View):
    def get(self, request, *args, **kwargs):
        token = request.GET.get("token")                    
        amount = request.GET.get("amount")                  
        o_id = request.GET.get("order_id")
		               

        url = "https://khalti.com/api/v2/payment/verify/"   

        payload = {                                 
            "token": token,
            "amount": amount
        }

        headers = {                                
            "Authorization":  Khalti.objects.get(id=1).private_key,
        }
		
        order_obj = PayHistory.objects.get(id=o_id)
        print(order_obj)
        response = requests.post(url, payload, headers=headers) 
        print(response)   
		     
        resp_dict = response.json()               
        if resp_dict.get("idx"):                 
            success = True                        
            order_obj.paid = True    
            order_obj.save()                      

        else:                                     
            success = False                         

        data = {
            "success": success
        }

        return JsonResponse(data)   

def logout(request):
	auth.logout(request)
	return redirect('/')


# Generated by Django 3.1 on 2022-04-13 09:35

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('username', models.CharField(default='', max_length=100, unique=True, verbose_name='Username')),
                ('email', models.EmailField(max_length=254, unique=True, verbose_name='email address')),
                ('first_name', models.CharField(max_length=200, null=True)),
                ('last_name', models.CharField(max_length=200, null=True)),
                ('gender', models.CharField(choices=[('Male', 'Male'), ('Female', 'Female')], default='', max_length=10)),
                ('photo', models.ImageField(blank=True, default='/static/images/profile1.png', null=True, upload_to='users')),
                ('country', models.CharField(max_length=200, null=True)),
                ('bio', models.TextField(blank=True, default='')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Membership',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(blank=True, null=True)),
                ('membership_type', models.CharField(choices=[('Extended', 'Extended'), ('Advanced', 'Advanced'), ('Medium', 'Medium'), ('Basic', 'Basic'), ('Free', 'Free')], default='Free', max_length=30)),
                ('duration', models.PositiveIntegerField(default=7)),
                ('duration_period', models.CharField(choices=[('Days', 'Days'), ('Week', 'Week'), ('Months', 'Months')], default='Day', max_length=100)),
                ('price', models.DecimalField(decimal_places=2, default=0.0, max_digits=10)),
            ],
        ),
        migrations.CreateModel(
            name='UserSetting',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('account_verified', models.BooleanField(default=False)),
                ('verified_code', models.CharField(blank=True, default='', max_length=100)),
                ('verification_expires', models.DateField(default=datetime.date(2022, 4, 16))),
                ('code_expired', models.BooleanField(default=False)),
                ('recieve_email_notice', models.BooleanField(default=True)),
                ('user', models.OneToOneField(default=None, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='UserMembership',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('reference_code', models.CharField(blank=True, default='', max_length=100)),
                ('membership', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='user_membership', to='app.membership')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='user_membership', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('expires_in', models.DateField(blank=True, null=True)),
                ('active', models.BooleanField(default=True)),
                ('user_membership', models.ForeignKey(default=None, on_delete=django.db.models.deletion.CASCADE, related_name='subscription', to='app.usermembership')),
            ],
        ),
        migrations.CreateModel(
            name='PayHistory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('paystack_charge_id', models.CharField(blank=True, default='', max_length=100)),
                ('paystack_access_code', models.CharField(blank=True, default='', max_length=100)),
                ('amount', models.DecimalField(decimal_places=2, default=0.0, max_digits=10)),
                ('paid', models.BooleanField(default=False)),
                ('date', models.DateTimeField(auto_now_add=True)),
                ('payment_for', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='app.membership')),
                ('user', models.ForeignKey(default=None, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# Generated by Django 3.1 on 2022-05-01 10:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shop', '0003_esewa'),
    ]

    operations = [
        migrations.CreateModel(
            name='Khalti',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('public_key', models.CharField(blank=True, max_length=150, null=True)),
                ('private_key', models.CharField(blank=True, max_length=150, null=True)),
            ],
        ),
    ]



# Generated by Django 3.1 on 2022-04-30 13:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('app', '0003_auto_20220430_1815'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Esewa',
        ),
        migrations.DeleteModel(
            name='Khalti',
        ),
    ]



from .models import Order
from django import forms

class CheckoutForm(forms.ModelForm):
    class Meta:
        model = Order
        fields = ["ordered_by","shipping_address","mobile", "email",'payment_method']


from django.contrib import admin

# Register your models here.
from .models import Product, Cart, CartProduct, Order, eSewa,Khalti
admin.site.register(Product)
admin.site.register(Cart)
admin.site.register(CartProduct)
admin.site.register(Order)
admin.site.register(eSewa)
admin.site.register(Khalti)



from rest_framework import serializers
from .models import User


class RegisterSerializer(serializers.ModelSerializer):
	class Meta:
		model = User
		fields = ['first_name', 'last_name', 'username', 'email', 'password']


from django.db import models
from app.models import User

# Create your models here.

# Product model
class Product(models.Model):
    title = models.CharField(max_length=100)
    image = models.ImageField(upload_to="products",null=True,blank=True)
    description = models.TextField(max_length=300,null=True,blank=True)
    price = models.IntegerField()
    # payment_completed = models.BooleanField(default=False, null=True, blank=True)

# this is for the display the title in the admin panel
    def __str__(self):
        return self.title




class eSewa(models.Model):
	 merchant_id = models.CharField(max_length=100,default='epay_payment')

class Khalti(models.Model):
    public_key = models.CharField(max_length=150,blank=True,null=True)
    private_key = models.CharField(max_length=150,blank=True,null=True)
    
 

class Cart(models.Model):
    customer= models.OneToOneField(User, on_delete= models.SET_NULL,null=True, blank= True)
    total = models.PositiveIntegerField(default=0)
    created_at = models.DateTimeField(auto_now_add= True)

    def __str__(self):
        return "Cart:" + str(self.id)


class CartProduct(models.Model):
    cart = models.ForeignKey(Cart, on_delete=models.CASCADE, default=0)
    product= models.ForeignKey(Product, on_delete= models.CASCADE)
    rate = models.PositiveBigIntegerField()
    quantity = models.PositiveBigIntegerField()
    subtotal = models.PositiveBigIntegerField()

    def __str__(self):
        return "Cart: " + str(self.cart.id) + "CartProduct:" + str(self.id)

ORDER_STATUS = (
    ("Order Received", "Order Received"),
    ("Order Processing", "Order Processing"),
    ("On the way", "On the way"),
    ("Order Completed", "Order Completed"),
    ("Order Canceled", "Order Canceled"),
)

METHOD = (
("Cash On Delivery","Cash On Delivery"),
("Esewa","Esewa"),
("Khalti", "Khalti"),
)

class Order(models.Model):
    cart = models.OneToOneField(Cart, on_delete=models.CASCADE)
    ordered_by = models.CharField(max_length=200)
    shipping_address = models.CharField(max_length=200)
    mobile = models.CharField(max_length=10)
    email = models.EmailField(null=True, blank= True)
    subtotal = models.PositiveBigIntegerField()
    discount = models.PositiveBigIntegerField()
    total = models.PositiveBigIntegerField()
    order_status = models.CharField(max_length=50, choices= ORDER_STATUS)
    created_at = models.DateTimeField(auto_now_add= True)
    payment_method = models.CharField(max_length=20,choices=METHOD,default="Cash On Delivery")
    payment_completed = models.BooleanField(default=False,null=True,blank=True)

    def __str__(self):
        return "Order:" + str(self.id)
 



# Generated by Django 3.1 on 2022-04-30 12:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('shop', '0001_initial'),
    ]

    operations = [
        migrations.DeleteModel(
            name='eSewa',
        ),
    ]



from django.http import HttpResponse
from django.shortcuts import render
from django.views.generic import View, TemplateView, CreateView, FormView, DetailView, ListView   
from django.shortcuts import get_object_or_404, render,redirect
from .models import *
from .forms import CheckoutForm
from django.urls import reverse_lazy
from django.http import JsonResponse
from django.urls import reverse
import requests

# Create your views here.

def product(request):
    return HttpResponse("hi there")


class HomePage(ListView):
    template_name = "shop/home.html"
    model = Product


class EsewaProductRequestView(View):
    def get(self,request,*args,**kwargs):
        # o_id = request.GET.get(kwargs['pk'])
        # print(o_id)
        # order = Product.objects.get(pk=o_id)
        # order = get_object_or_404(Product, id=kwargs['pk'])
        # context = {
        #     "order":order
        # }
        o_id = request.GET.get("o_id")
        order = Order.objects.get(id= o_id)
        print(order)
        esewa = eSewa.objects.get(id=1)
        context ={
            "order":order,
            'esewa':esewa
        }
        return render(request,"shop/esewarequest.html",context)


class EsewaProductVerifyView(View):
    def get(self, request, *args, **kwargs):

        import xml.etree.ElementTree as ET                  

        oid = request.GET.get("oid")                        
        amt = request.GET.get("amt")                        
        refId = request.GET.get("refId")                    

        url = "https://uat.esewa.com.np/epay/transrec"      

        d = {                                               
            'amt': amt,
            'scd': eSewa.objects.get(id=1).merchant_id,
            'rid': refId,
            'pid': oid,
        }
        resp = requests.post(url, d)                        

        root = ET.fromstring(resp.content)                                                 

        status = root[0].text.strip()                       

        order_id = oid 
         
        order_obj = Order.objects.get(id=order_id)

        if status == "Success":                             
            order_obj.payment_completed = True
            order_obj.save()
            return redirect("/")
        else:                                               
            return redirect("/esewaproduct-request/?o_id="+order_id)


def detail(request):
    return render(request,'detail.html')

class AddToCartView(TemplateView):
    template_name="shop/addtocart.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)  
        # get product id from requested url
        product_id = self.kwargs['pro_id']
        
        #get product
        product_obj = Product.objects.get(id=product_id)
        #check whether cart exists
        cart_id = self.request.session.get("cart_id",None)
        if cart_id:
            cart_obj= Cart.objects.get(id= cart_id)
            this_product_in_cart = cart_obj.cartproduct_set.filter(product=product_obj)
            #where items already exist in cart
            if this_product_in_cart.exists():
                cartproduct = this_product_in_cart.last()
                cartproduct.quantity += 1
                cartproduct.subtotal += product_obj.price
                cartproduct.save()
                cart_obj.total += product_obj.price
                cart_obj.save()
            else:
                # new items added in cart
                cartproduct= CartProduct.objects.create(cart=cart_obj, product=product_obj, rate=product_obj.price, quantity=1, subtotal=product_obj.price)
                cart_obj.total += product_obj.price
                cart_obj.save()
        else:
            cart_obj=Cart.objects.create(total=0)
            self.request.session['cart_id'] = cart_obj.id
            cartproduct = CartProduct.objects.create(cart= cart_obj, product=product_obj, rate=product_obj.price, quantity= 1, subtotal=product_obj.price)
            cart_obj.total += product_obj.price
            cart_obj.save()
        #check if product already exists in cart

        return context


class EmptyCartView (View):
    def get(self, request, *args, **kwargs ):
        cart_id = request.session.get("cart_id",None)
        if cart_id:
            cart = Cart.objects.get(id=cart_id)
            cart.cartproduct_set.all().delete()
            cart.total = 0
            cart.save()
        return redirect("mycart")


class MyCartView(TemplateView):
    template_name="shop/mycart.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        cart_id=self.request.session.get("cart_id", None)
        if cart_id:
            cart=Cart.objects.get(id=cart_id)
        else:
            cart=None
        
        context['cart']=cart
        return context 

class ManageCartView(View):
    def get(self, request, *args, **kwargs):
        
        cp_id = self.kwargs["cp_id"]
        action = request.GET.get("action")
        cp_obj = CartProduct.objects.get(id=cp_id)
        cart_obj = cp_obj.cart
        if action == "inc":
            cp_obj.quantity += 1
            cp_obj.subtotal += cp_obj.rate
            cp_obj.save()
            cart_obj.total += cp_obj.rate
            cart_obj.save()
        elif action == "dcr":
            cp_obj.quantity -= 1
            cp_obj.subtotal -= cp_obj.rate
            cp_obj.save()
            cart_obj.total -= cp_obj.rate
            cart_obj.save()
            if cp_obj.quantity ==0 :
                cp_obj.delete()

        elif action == "rmv":
            cart_obj.total -= cp_obj.subtotal
            cart_obj.save()
            cp_obj.delete()
        return redirect("mycart")

class CheckoutView(CreateView):
    template_name="shop/checkout.html"
    form_class= CheckoutForm
    success_url = reverse_lazy("home")

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        cart_id  = self.request.session.get("cart_id",None)
        if cart_id:
            cart_obj = Cart.objects.get(id=cart_id)
        else:
            cart_obj = None
        context['cart'] = cart_obj
        return context
    def form_valid(self, form):
        cart_id= self.request.session.get("cart_id")
        if cart_id:
            cart_obj = Cart.objects.get(id=cart_id)
            form.instance.cart = cart_obj
            form.instance.subtotal = cart_obj.total
            form.instance.discount = 0 
            form.instance.total = cart_obj.total
            form.instance.order_status = "Order Received"
            del self.request.session['cart_id']
            pm = form.cleaned_data.get("payment_method")
            order = form.save()
            if pm == "Esewa":
                return redirect(reverse("esewarequest")+ "?o_id=" + str(order.id))
            elif pm == "Khalti":                              
                return redirect(reverse("khaltiproductrequest") + "?o_id=" + str(order.id))
        else:
            return redirect("home")
        return super().form_valid(form)



class KhaltiProductRequestView(View):
    def get(self, request, *args, **kwargs):
        o_id = request.GET.get("o_id")
        order = Order.objects.get(id=o_id)
        khalti = Khalti.objects.get(id=1)
        context = {
            "order": order,
            'khalti': khalti

        }
        return render(request, "shop/khaltirequest.html", context)


class KhaltiProductVerifyView(View):
    def get(self, request, *args, **kwargs):
        token = request.GET.get("token")                    
        amount = request.GET.get("amount")                  
        o_id = request.GET.get("order_id")                  

        url = "https://khalti.com/api/v2/payment/verify/"   

        payload = {                                 
            "token": token,
            "amount": amount
        }

        headers = {                                
            "Authorization": Khalti.objects.get(id=1).private_key,
        }

        order_obj = Order.objects.get(id=o_id)
        print(order_obj)
        response = requests.post(url, payload, headers=headers) 
        print(response)        
        resp_dict = response.json()  
        print(resp_dict)            

        if resp_dict.get("idx"):                 
            success = True                        
            order_obj.payment_completed = True    
            order_obj.save()                      

        else:                                     
            success = False                         

        data = {
            "success": success
        }

        return JsonResponse(data)          



from django.urls import path
from .views import *
from django.contrib.auth.decorators import login_required


urlpatterns = [
     
    # path('',product,name='product'),
    path('',login_required(HomePage.as_view()),name="home"),
    path("esewa-product-request/", login_required(EsewaProductRequestView.as_view()), name="esewarequest"),
    path("esewaproduct-verify/", login_required(EsewaProductVerifyView.as_view()), name="esewaproductverify"),
    path("khaltiproduct-request/", login_required(KhaltiProductRequestView.as_view()), name="khaltiproductrequest"),
    #this is the url for verifying whether khalti payment is success or not
    path("khaltiproduct-verify/", login_required(KhaltiProductVerifyView.as_view()), name="khaltiproductverify"),
    path("add-to-cart-<int:pro_id>/", login_required(AddToCartView.as_view()), name="addtocart"),
    path("my-cart/", login_required(MyCartView.as_view()), name="mycart"),
    path("manage-cart/<int:cp_id>/", login_required(ManageCartView.as_view()), name="managecart"),
    path("empty-cart/", login_required(EmptyCartView.as_view()), name="emptycart"),
    path("checkout/", login_required(CheckoutView.as_view()), name="checkout"),
]



from django.urls import path
from . import views
from .views import Register, Login,EsewaVerifyView,KhaltiVerifyView

urlpatterns = [
	path('', views.home, name='index'),
	path('login/', views.signin, name='login'),
	path('home/', views.index, name='home'),
    path('logout/', views.logout, name='logout'),
    path('check-mail-ajax/', views.check_mail_ajax, name='check_mail_ajax'),
    path('register/', Register.as_view(), name='register'),
    path('login-req', Login.as_view(), name='login_ajax'),
    path('esewa-verify/', EsewaVerifyView.as_view(), name="esewaverify"),

    path('subscription/', views.subscription, name='subscription'),
    path('subscribe/', views.subscribe, name='subscribe'),
    path('subscribed/', views.subscribed, name='subscribed'),
    path('sub/', views.end_sub, name='sub'),
    path("khalti-request/", views.khaltirequest, name="khaltirequest"),
    #this is the url for verifying whether khalti payment is success or not
    path("khalti-verify/", KhaltiVerifyView.as_view(), name="khaltiverify"),

]


# Generated by Django 3.1 on 2022-04-25 11:03

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Cart',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('total', models.PositiveIntegerField(default=0)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('customer', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='eSewa',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('merchant_id', models.CharField(default='epay_payment', max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='Product',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100)),
                ('image', models.ImageField(blank=True, null=True, upload_to='products')),
                ('description', models.TextField(blank=True, max_length=300, null=True)),
                ('price', models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ordered_by', models.CharField(max_length=200)),
                ('shipping_address', models.CharField(max_length=200)),
                ('mobile', models.CharField(max_length=10)),
                ('email', models.EmailField(blank=True, max_length=254, null=True)),
                ('subtotal', models.PositiveBigIntegerField()),
                ('discount', models.PositiveBigIntegerField()),
                ('total', models.PositiveBigIntegerField()),
                ('order_status', models.CharField(choices=[('Order Received', 'Order Received'), ('Order Processing', 'Order Processing'), ('On the way', 'On the way'), ('Order Completed', 'Order Completed'), ('Order Canceled', 'Order Canceled')], max_length=50)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('payment_method', models.CharField(choices=[('Cash On Delivery', 'Cash On Delivery'), ('Esewa', 'Esewa'), ('Khalti', 'Khalti')], default='Cash On Delivery', max_length=20)),
                ('payment_completed', models.BooleanField(blank=True, default=False, null=True)),
                ('cart', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='shop.cart')),
            ],
        ),
        migrations.CreateModel(
            name='CartProduct',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('rate', models.PositiveBigIntegerField()),
                ('quantity', models.PositiveBigIntegerField()),
                ('subtotal', models.PositiveBigIntegerField()),
                ('cart', models.ForeignKey(default=0, on_delete=django.db.models.deletion.CASCADE, to='shop.cart')),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shop.product')),
            ],
        ),
    ]



# Generated by Django 3.1 on 2022-04-14 07:15

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('app', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='payhistory',
            name='paystack_access_code',
        ),
        migrations.RemoveField(
            model_name='payhistory',
            name='paystack_charge_id',
        ),
        migrations.AlterField(
            model_name='usersetting',
            name='verification_expires',
            field=models.DateField(default=datetime.date(2022, 4, 17)),
        ),
    ]



# Generated by Django 3.1 on 2022-04-30 13:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shop', '0002_delete_esewa'),
    ]

    operations = [
        migrations.CreateModel(
            name='eSewa',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('merchant_id', models.CharField(default='epay_payment', max_length=100)),
            ],
        ),
    ]



# Generated by Django 3.1 on 2022-05-01 10:01

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('app', '0004_auto_20220430_1940'),
    ]

    operations = [
        migrations.AlterField(
            model_name='usersetting',
            name='verification_expires',
            field=models.DateField(default=datetime.date(2022, 5, 4)),
        ),
    ]



from setuptools import setup

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
with open(this_directory / "README.md", encoding="utf8", errors='ignore') as f:
    long_description = f.read()

setup(
    name='paradox_pay',
    # other arguments omitted
    long_description=long_description,
    long_description_content_type='text/markdown'
)


from django.contrib import admin
from .models import User, Membership, UserMembership, Subscription,PayHistory

admin.site.register(User)
admin.site.register(UserMembership)
admin.site.register(Membership)
admin.site.register(Subscription)
admin.site.register(PayHistory)


from django.db import models
from django.contrib.auth.base_user import BaseUserManager
# from django.utils.translation import ugettext_lazy as _
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.models import AbstractUser
from django.db.models.signals import post_save, pre_save
from django.dispatch import receiver
from django.conf import settings
import datetime
from datetime import timedelta
from datetime import datetime as dt
today = datetime.date.today()
### Custom User Model Used Here

class UserManager(BaseUserManager):
    """
    Custom user model manager where email is the unique identifiers
    for authentication instead of username.
    """
    def create_user(self, email, password, **extra_fields):
        """
        Create and save a User with the given email and password.
        """
        if not email:
            raise ValueError(_('The Email must be set'))
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save()
        return user

    def create_superuser(self, email, password, **extra_fields):
        """
        Create and save a SuperUser with the given email and password.
        """
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)
        extra_fields.setdefault('is_active', True)

        if extra_fields.get('is_staff') is not True:
            raise ValueError(_('Superuser must have is_staff=True.'))
        if extra_fields.get('is_superuser') is not True:
            raise ValueError(_('Superuser must have is_superuser=True.'))
        return self.create_user(email, password, **extra_fields)


#### This is User Profile
class User(AbstractUser):
    user_gender = (
        ('Male', 'Male'),
        ('Female', 'Female')
    )
    username = models.CharField(_('Username'), max_length=100, default='', unique=True)
    email = models.EmailField(_('email address'), unique=True)
    first_name = models.CharField(max_length=200, null=True)
    last_name = models.CharField(max_length=200, null=True)
    gender = models.CharField(max_length=10, default='', choices=user_gender)
    # mobile = models.CharField(max_length=200, null=True)
    photo = models.ImageField(upload_to='users', default="/static/images/profile1.png", null=True, blank=True)
    country = models.CharField(max_length=200, null=True)
    bio = models.TextField(default='', blank=True)

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ['first_name', 'last_name', 'username']

    objects = UserManager()

    def __str__(self):
        return self.first_name + ' ' + self.last_name


#### This is user settings
class UserSetting(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE, default=None)
    account_verified = models.BooleanField(default=False)
    verified_code = models.CharField(max_length=100, default='', blank=True)
    verification_expires = models.DateField(default=dt.now().date() + timedelta(days=settings.VERIFY_EXPIRE_DAYS))
    code_expired = models.BooleanField(default=False)
    recieve_email_notice = models.BooleanField(default=True)

#### User Payment History
 
class PayHistory(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE, default=None)
    # paystack_charge_id = models.CharField(max_length=100, default='', blank=True)
    # paystack_access_code = models.CharField(max_length=100, default='', blank=True)
    payment_for = models.ForeignKey('Membership', on_delete=models.SET_NULL, null=True)
    amount = models.DecimalField(max_digits=10, decimal_places=2, default=0.00)
    paid = models.BooleanField(default=False)
    date = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return self.user.username

#### Membership
class Membership(models.Model):
    MEMBERSHIP_CHOICES = (
    	('Extended', 'Extended'), # Note that they are all capitalize//
    	('Advanced', 'Advanced'),
    	('Medium', 'Medium'),
        ('Basic', 'Basic'),
        ('Free', 'Free')
    )
    PERIOD_DURATION = (
        ('Days', 'Days'),
        ('Week', 'Week'),
        ('Months', 'Months'),
    )
    slug = models.SlugField(null=True, blank=True)
    membership_type = models.CharField(choices=MEMBERSHIP_CHOICES, default='Free', max_length=30)
    duration = models.PositiveIntegerField(default=7)
    duration_period = models.CharField(max_length=100, default='Day', choices=PERIOD_DURATION)
    price = models.DecimalField(max_digits=10, decimal_places=2, default=0.00)

    def __str__(self):
       return self.membership_type

#### User Membership
class UserMembership(models.Model):
    user = models.OneToOneField(User, related_name='user_membership', on_delete=models.CASCADE)
    membership = models.ForeignKey(Membership, related_name='user_membership', on_delete=models.SET_NULL, null=True)
    reference_code = models.CharField(max_length=100, default='', blank=True)

    def __str__(self):
       return self.user.username

@receiver(post_save, sender=UserMembership)
def create_subscription(sender, instance, *args, **kwargs):
	if instance:
		Subscription.objects.create(user_membership=instance, expires_in=dt.now().date() + timedelta(days=instance.membership.duration))


#### User Subscription
class Subscription(models.Model):
    user_membership = models.ForeignKey(UserMembership, related_name='subscription', on_delete=models.CASCADE, default=None)
    expires_in = models.DateField(null=True, blank=True)
    active = models.BooleanField(default=True)

    def __str__(self):
      return self.user_membership.user.username

@receiver(post_save, sender=Subscription)
def update_active(sender, instance, *args, **kwargs):
	if instance.expires_in < today:
		subscription = Subscription.objects.get(id=instance.id)
		subscription.delete()